#!/usr/bin/env python

import os
import time
import sys
import BaseHTTPServer

from optparse import OptionParser
from os.path import join as pjoin
sys.path.append(pjoin(os.path.dirname(__file__), '../'))

from yubico.yubico import BAD_STATUS_CODES

mock_action = None
signature = None


class Handler(BaseHTTPServer.BaseHTTPRequestHandler):

    def do_GET(self):
        global mock_action, signature

        if self.path.find('?') != -1:
            self.path, self.query_string = self.path.split('?', 1)
            split = self.query_string.split('&')
            self.query_string = dict([pair.split('=', 1) for pair in split])

        else:
            self.query_string = {}

        if self.path == '/set_mock_action':
            action = self.query_string['action']

            if 'signature' in self.query_string:
                signature = self.query_string['signature']
            else:
                signature = None

            print 'Setting mock_action to %s' % (action)
            mock_action = action
            self._end(status_code=200)
            return

        if mock_action in BAD_STATUS_CODES:
            return self._send_status(status=mock_action)
        elif mock_action == 'no_such_client':
            return self._send_status(status='NO_SUCH_CLIENT')
        elif mock_action == 'no_signature_ok':
            return self._send_status(status='OK')
        elif mock_action == 'ok_signature':
            return self._send_status(status='OK',
                                     signature=signature)
        elif mock_action == 'no_signature_ok_invalid_otp_in_response':
            return self._send_status(status='OK',
                                     signature=signature, otp='different')
        elif mock_action == 'no_signature_ok_invalid_nonce_in_response':
            return self._send_status(status='OK',
                                     signature=signature, nonce='different')
        elif mock_action == 'timeout':
            time.sleep(1)
            return self._send_status(status='OK')
        else:
            self._end(status_code=500)
            return

    def _end(self, status_code=200, body=''):
        print 'Sending response: status_code=%s, body=%s' % (status_code, body)
        self.send_response(status_code)
        self.send_header('Content-Type', 'text/plain')
        self.send_header('Content-Length', str(len(body)))
        self.end_headers()
        self.wfile.write(body)

    def _send_status(self, status, signature=None, otp=None, nonce=None):
        if signature:
            body = '\nh=%s\nstatus=%s' % (signature, status)
        else:
            body = 'status=%s' % (status)

        if otp:
            body += '&otp=%s' % (otp)

        if nonce:
            body += '&nonce=%s' % (nonce)

        self._end(body=body)


def main():
    usage = 'usage: %prog --port=<port>'
    parser = OptionParser(usage=usage)
    parser.add_option('--port', dest='port', default=8881,
                      help='Port to listen on', metavar='PORT')

    (options, args) = parser.parse_args()

    server_class = BaseHTTPServer.HTTPServer
    httpd = server_class(('127.0.0.1', int(options.port)), Handler)
    print 'Mock API server listening on 127.0.0.1:%s' % (options.port)

    try:
        httpd.serve_forever()
    except KeyboardInterrupt:
        pass

    httpd.server_close()

main()



__all___ = [
    'YubicoError',
    'StatusCodeError',
    'InvalidClientIdError',
    'InvalidValidationResponse',
    'SignatureVerificationError'
]


class YubicoError(Exception):
    """ Base class for Yubico related exceptions. """
    pass


class StatusCodeError(YubicoError):
    def __init__(self, status_code):
        self.status_code = status_code

    def __str__(self):
        return ('Yubico server returned the following status code: %s' %
                (self.status_code))


class InvalidClientIdError(YubicoError):
    def __init__(self, client_id):
        self.client_id = client_id

    def __str__(self):
        return 'The client with ID %s does not exist' % (self.client_id)


class InvalidValidationResponse(YubicoError):
    def __init__(self, reason, response, parameters=None):
        self.reason = reason
        self.response = response
        self.parameters = parameters
        self.message = self.reason

    def __str__(self):
        return self.reason


class SignatureVerificationError(YubicoError):
    def __init__(self, generated_signature, response_signature):
        self.generated_signature = generated_signature
        self.response_signature = response_signature

    def __str__(self):
        return repr('Server response message signature verification failed' +
                    '(expected %s, got %s)' % (self.generated_signature,
                                               self.response_signature))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import re
import sys
import logging

from glob import glob
from os.path import splitext, basename, join as pjoin
from unittest import TextTestRunner, TestLoader

from setuptools import setup
from distutils.core import Command

sys.path.insert(0, pjoin(os.path.dirname(__file__)))
from tests.utils import MockAPIServerRunner

TEST_PATHS = ['tests']

version_re = re.compile(
    r'__version__ = (\(.*?\))')

cwd = os.path.dirname(os.path.abspath(__file__))
fp = open(os.path.join(cwd, 'yubico', '__init__.py'))

version = None
for line in fp:
    match = version_re.search(line)
    if match:
        version = eval(match.group(1))
        break
else:
    raise Exception('Cannot find version in __init__.py')
fp.close()


class TestCommand(Command):
    description = 'run test suite'
    user_options = []

    def initialize_options(self):
        FORMAT = '%(asctime)-15s [%(levelname)s] %(message)s'
        logging.basicConfig(format=FORMAT)
        
        THIS_DIR = os.path.abspath(os.path.split(__file__)[0])
        sys.path.insert(0, THIS_DIR)
        for test_path in TEST_PATHS:
            sys.path.insert(0, pjoin(THIS_DIR, test_path))
        self._dir = os.getcwd()

    def finalize_options(self):
        pass

    def run(self):
        self._run_mock_api_server()
        status = self._run_tests()
        sys.exit(status)

    def _run_tests(self):
        testfiles = []
        for test_path in TEST_PATHS:
            for t in glob(pjoin(self._dir, test_path, 'test_*.py')):
                testfiles.append('.'.join(
                    [test_path.replace('/', '.'), splitext(basename(t))[0]]))

        tests = TestLoader().loadTestsFromNames(testfiles)

        t = TextTestRunner(verbosity=2)
        res = t.run(tests)
        return not res.wasSuccessful()

    def _run_mock_api_server(self):
        for port in [8881, 8882, 8883]:
            server = MockAPIServerRunner(port=port)
            server.setUp()


setup(name='yubico',
      version='.' . join(map(str, version)),
      description='Python Yubico Client',
      author='Tomaz Muraus',
      author_email='tomaz+pypi@tomaz.me',
      license='BSD',
      url='http://github.com/Kami/python-yubico-client/',
      download_url='http://github.com/Kami/python-yubico-client/downloads/',
      packages=['yubico'],
      provides=['yubico'],
      install_requires=[
        'requests == 1.1.0',
      ],
      cmdclass={
          'test': TestCommand,
      },
      classifiers=[
          'Development Status :: 4 - Beta',
          'Environment :: Console',
          'Intended Audience :: Developers',
          'Intended Audience :: System Administrators',
          'License :: OSI Approved :: BSD License',
          'Operating System :: OS Independent',
          'Programming Language :: Python',
          'Topic :: Security',
          'Topic :: Software Development :: Libraries :: Python Modules',
    ]
)
